package net.javajigi.abs.model;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

import net.javajigi.db.ConnectionManager;
import net.javajigi.util.DateTime;

public class ScheduleDAO {
	public int createPlaneSchedule(PlaneSchedule planeSchedule)
		throws SQLException {
		Connection con = null;
		PreparedStatement pstmt = null;
		try {
			StringBuffer createQuery = new StringBuffer();
			createQuery.append("INSERT INTO PLANESCHEDULE VALUES");
			createQuery.append("(?, ?, ?)");

			con = ConnectionManager.getConnection();
			pstmt = con.prepareStatement(createQuery.toString());
			pstmt.setString(1, planeSchedule.getScheduleDate());
			pstmt.setInt(2, planeSchedule.getScheduleCode());
			pstmt.setInt(3, planeSchedule.getRestNo());

			return pstmt.executeUpdate();
		} finally {
			if (pstmt != null) {
				pstmt.close();
			}
			if (con != null) {
				con.close();
			}
		}
	}

	public int modifyPlaneSchedule(PlaneSchedule planeSchedule)
		throws SQLException {
		Connection con = null;
		PreparedStatement pstmt = null;
		try {
			StringBuffer modifyQuery = new StringBuffer();
			modifyQuery.append("UPDATE PLANESCHEDULE ");
			modifyQuery.append("SET restno = restno-");
			modifyQuery.append(planeSchedule.getReserveSeatNo());
			modifyQuery.append(" WHERE ");
			modifyQuery.append("scheduledate=? AND schedulecode=?");

			con = ConnectionManager.getConnection();
			pstmt = con.prepareStatement(modifyQuery.toString());
			pstmt.setString(1, planeSchedule.getScheduleDate());
			pstmt.setInt(2, planeSchedule.getScheduleCode());

			return pstmt.executeUpdate();
		} finally {
			if (pstmt != null) {
				pstmt.close();
			}
			if (con != null) {
				con.close();
			}
		}
	}

	/**
	 * Ư װ ܿ¼ ã ޽.
	 * PLANESCHEDULE̺ ܿ¼   ,
	 * AIRPLANE̺ װ ¼ ܿ¼ ȴ. 
	 */
	public PlaneSchedule getPlaneSchedule(String date, int scheduleCode)
		throws SQLException {
		Connection con = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			StringBuffer selectQuery = new StringBuffer();
			selectQuery.append("SELECT ");
			selectQuery.append("restno ");
			selectQuery.append("FROM PLANESCHEDULE WHERE ");
			selectQuery.append("scheduledate=? and schedulecode=?");

			con = ConnectionManager.getConnection();
			pstmt = con.prepareStatement(selectQuery.toString());
			pstmt.setString(1, date);
			pstmt.setInt(2, scheduleCode);

			rs = pstmt.executeQuery();

			PlaneSchedule planeSchedule = new PlaneSchedule();
			planeSchedule.setScheduleDate(date);
			planeSchedule.setScheduleCode(scheduleCode);
			if (rs.next()) {
				planeSchedule.setRestNo(rs.getInt("restno"));
			} else {
				int airplaneCode = getAirplaneCode(scheduleCode);
				planeSchedule.setRestNo(getSeatNo(airplaneCode));
			}

			return planeSchedule;
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
			if (con != null) {
				con.close();
			}
		}
	}

	/**
	 * װ ¼ ϴ ޽.
	 */
	public int getSeatNo(int airplaneCode) throws SQLException {
		Connection con = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			StringBuffer selectQuery = new StringBuffer();
			selectQuery.append("SELECT ");
			selectQuery.append("seatno ");
			selectQuery.append("FROM AIRPLANE WHERE ");
			selectQuery.append("airplanecode=?");

			con = ConnectionManager.getConnection();
			pstmt = con.prepareStatement(selectQuery.toString());
			pstmt.setInt(1, airplaneCode);

			rs = pstmt.executeQuery();

			if (rs.next()) {
				return rs.getInt("seatno");
			} else {
				return 0;
			}
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
			if (con != null) {
				con.close();
			}
		}
	}

	/**
	 * ߵÿ ñ װ ϴ ޽.
	 */
	public String getAirplaneFee(String startCity, String endCity)
		throws SQLException {
		Connection con = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			StringBuffer selectQuery = new StringBuffer();
			selectQuery.append("SELECT ");
			selectQuery.append("fee ");
			selectQuery.append("FROM AIRPLANEFEE WHERE ");
			selectQuery.append("startcity=? and endcity=?");

			con = ConnectionManager.getConnection();
			pstmt = con.prepareStatement(selectQuery.toString());
			pstmt.setString(1, startCity);
			pstmt.setString(2, endCity);

			rs = pstmt.executeQuery();

			if (rs.next()) {
				return rs.getString("fee");
			} else {
				return null;
			}
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
			if (con != null) {
				con.close();
			}
		}
	}

	/**
	 *  װ ̿ ⸦ ã Ͽ 
	 *  ڵ  ϴ ޽.  
	 */
	public int getAirplaneCode(int scheduleCode) throws SQLException {
		Connection con = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			StringBuffer selectQuery = new StringBuffer();
			selectQuery.append("SELECT ");
			selectQuery.append("airplanecode ");
			selectQuery.append("FROM SCHEDULECODE WHERE ");
			selectQuery.append("schedulecode=?");

			con = ConnectionManager.getConnection();
			pstmt = con.prepareStatement(selectQuery.toString());
			pstmt.setInt(1, scheduleCode);

			rs = pstmt.executeQuery();

			if (rs.next()) {
				return rs.getInt("airplanecode");
			} else {
				return 0;
			}
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
			if (con != null) {
				con.close();
			}
		}
	}

	/**
	 * Ư װ  ã ִ ޽.
	 *   ScheduleInfoŬ Ǿ ȯ.
	 */
	public ScheduleInfo getScheduleInfo(int scheduleCode, String date)
		throws SQLException {

		Connection con = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			StringBuffer selectQuery = new StringBuffer();
			selectQuery.append("SELECT ");
			selectQuery.append("weekday, startcity, starttime, arrivecity, ");
			selectQuery.append("arrivetime, airline, airplanecode ");
			selectQuery.append("FROM SCHEDULECODE WHERE ");
			selectQuery.append("schedulecode=?");

			con = ConnectionManager.getConnection();
			pstmt = con.prepareStatement(selectQuery.toString());
			pstmt.setInt(1, scheduleCode);

			rs = pstmt.executeQuery();

			if (rs.next()) {
				ScheduleInfo scheduleInfo = new ScheduleInfo();
				String startCity = rs.getString("startcity");
				String arriveCity = rs.getString("arrivecity");

				scheduleInfo.setScheduleCode(scheduleCode);
				scheduleInfo.setDate(date);
				scheduleInfo.setDay(rs.getString("weekday"));
				scheduleInfo.setStartCity(startCity);
				scheduleInfo.setStartTime(rs.getString("starttime"));
				scheduleInfo.setArriveCity(arriveCity);
				scheduleInfo.setArriveTime(rs.getString("arrivetime"));
				scheduleInfo.setAirLine(rs.getString("airline"));
				scheduleInfo.setAirPlaneCode(rs.getInt("airplanecode"));

				scheduleInfo.setFee(getAirplaneFee(startCity, arriveCity));
				scheduleInfo.setPlaneSchedule(
					getPlaneSchedule(date, scheduleCode));

				return scheduleInfo;
			} else {
				return null;
			}
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
			if (con != null) {
				con.close();
			}
		}
	}

	/**
	 * װ ˻  , ð, ߵ, ÿ شϴ
	 * װ ã ޽.
	 *  װ  ScheduleInfoü   List Ǿ ȯ.
	 */
	public List getScheduleInfos(
		String date,
		int time,
		String startCity,
		String arriveCity)
		throws SQLException, ParseException {
		Connection con = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;

		String weekDay = getWeekDay(date);

		try {
			StringBuffer selectQuery = new StringBuffer();
			selectQuery.append("SELECT ");
			selectQuery.append(
				"schedulecode, starttime, arrivetime, airline, airplanecode ");
			selectQuery.append("FROM SCHEDULECODE WHERE ");
			selectQuery.append("weekday=? and startcity=? and arrivecity=? ");
			selectQuery.append("and starttime >= ? and starttime < ?");

			con = ConnectionManager.getConnection();
			pstmt = con.prepareStatement(selectQuery.toString());

			pstmt.setString(1, weekDay);
			pstmt.setString(2, startCity);
			pstmt.setString(3, arriveCity);
			pstmt.setString(4, getTime(time));
			pstmt.setString(5, getTime(time + 2));

			rs = pstmt.executeQuery();

			List results = new ArrayList();

			while (rs.next()) {
				ScheduleInfo scheduleInfo = new ScheduleInfo();

				int scheduleCode = rs.getInt("schedulecode");
				scheduleInfo.setScheduleCode(scheduleCode);
				scheduleInfo.setDay(weekDay);
				scheduleInfo.setStartTime(rs.getString("starttime"));
				scheduleInfo.setArriveTime(rs.getString("arrivetime"));
				scheduleInfo.setAirLine(rs.getString("airline"));
				scheduleInfo.setAirPlaneCode(rs.getInt("airplanecode"));

				scheduleInfo.setFee(getAirplaneFee(startCity, arriveCity));
				scheduleInfo.setPlaneSchedule(
					getPlaneSchedule(date, scheduleCode));

				results.add(scheduleInfo);
			}

			return results;
		} finally {
			if (pstmt != null) {
				pstmt.close();
			}
			if (con != null) {
				con.close();
			}
		}
	}

	private String getWeekDay(String date) throws ParseException {
		int year = Integer.parseInt(date.substring(0, 4));
		int month = Integer.parseInt(date.substring(4, 6));
		int day = Integer.parseInt(date.substring(6, 8));
		String weekDay = DateTime.getDayOfWeek(year, month, day);
		return weekDay;
	}

	private String getTime(int time) {
		if (time < 10) {
			return "0" + time + "00";
		} else {
			return time + "00";
		}
	}
}
